@@ -1,26 +1,32 @@
 /*
- *  Copyright © 1998, 2003 Jonathan Blandford <jrb@mit.edu>
- *  Copyright © 2007 Christian Persch
+ * Copyright © 1998, 2003 Jonathan Blandford <jrb@mit.edu>
+ * Copyright © 2007 Christian Persch
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2, or (at your option)
- *  any later version.
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef AISLERIOT_BOARD_H
 #define AISLERIOT_BOARD_H
 
 #include <gtk/gtk.h>
+
+#ifdef HAVE_CLUTTER
+#include <clutter-gtk/clutter-gtk.h>
+#endif
+
+#include <libgames-support/games-card-theme.h>
+
 #include "game.h"
 
 G_BEGIN_DECLS
@@ -37,14 +43,22 @@ typedef struct _AisleriotBoardPrivate	AisleriotBoardPrivate;
 typedef struct _AisleriotBoardClass	AisleriotBoardClass;
 
 struct _AisleriotBoard {
+#ifdef HAVE_CLUTTER
+  GtkClutterEmbed parent_instance;
+#else
   GtkDrawingArea parent_instance;
+#endif
 
   /*< private >*/
   AisleriotBoardPrivate *priv;
 };
 
 struct _AisleriotBoardClass {
+#ifdef HAVE_CLUTTER
+  GtkClutterEmbedClass parent_class;
+#else
   GtkDrawingAreaClass parent_class;
+#endif
 
   /* keybinding signals */
   gboolean (* move_cursor)  (AisleriotBoard *board,
@@ -58,13 +72,12 @@ struct _AisleriotBoardClass {
 
 GType aisleriot_board_get_type (void);
 
-GtkWidget *aisleriot_board_new (AisleriotGame * game,
-                                gboolean scalable_cards);
+GtkWidget *aisleriot_board_new (AisleriotGame *game);
 
-gboolean aisleriot_board_set_card_theme (AisleriotBoard * board,
-                                         const char *card_theme);
+void aisleriot_board_set_card_theme (AisleriotBoard * board,
+                                     GamesCardTheme *theme);
 
-const char *aisleriot_board_get_card_theme (AisleriotBoard * board);
+GamesCardTheme *aisleriot_board_get_card_theme (AisleriotBoard * board);
 
 void aisleriot_board_set_click_to_move (AisleriotBoard * board,
                                         gboolean click_to_move);
