@@ -387,7 +387,7 @@ class UI(ui.UIFeedback):
         try:
             p = chess.pgn.PGN(path, 1)
         except chess.pgn.Error, e:
-            return e.message
+            return str(e)
         except IOError, e:
             return e.strerror
         
